"use strict";

exports.__esModule = true;
exports.resetLocalStateForTests = exports.addHook = exports.SingletonContainer = void 0;

var _react = _interopRequireWildcard(require("react"));

var _SingleItemContainer = require("./SingleItemContainer");

var _env = require("../utils/env");

var _warning = require("../utils/warning");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var singletonContainerMounted = false;
var singletonContainerRendered = false;
var singletonContainerMountedAutomatically = false;
var mountQueue = [];

var mountIntoContainerDefault = function mountIntoContainerDefault(item) {
  mountQueue.push(item);
};

var mountIntoContainer = mountIntoContainerDefault;

var SingletonContainer = function SingletonContainer() {
  singletonContainerRendered = true;
  (0, _react.useEffect)(function () {
    if (singletonContainerMounted) {
      (0, _warning.warning)('SingletonContainer is mounted second time. ' + 'You should mount SingletonContainer before any other component and never unmount it.' + 'Alternatively, dont use SingletonContainer it at all, we will handle that for you.');
    }

    singletonContainerMounted = true;
    return function () {
      return process.env.NODE_ENV !== 'test' && (0, _warning.warning)('SingletonContainer is removed from DOM. its not supported, singleton hooks will stop updating.');
    };
  }, []);

  var _useState = (0, _react.useState)([]),
      hooks = _useState[0],
      setHooks = _useState[1];

  (0, _react.useEffect)(function () {
    mountIntoContainer = function mountIntoContainer(item) {
      return setHooks(function (hooks) {
        return [].concat(hooks, [item]);
      });
    };

    setHooks(mountQueue);
  }, []);
  return _react["default"].createElement(_react["default"].Fragment, null, hooks.map(function (h, i) {
    return _react["default"].createElement(_SingleItemContainer.SingleItemContainer, _extends({}, h, {
      key: i
    }));
  }));
};

exports.SingletonContainer = SingletonContainer;

var addHook = function addHook(hook) {
  if (!singletonContainerRendered && !singletonContainerMountedAutomatically) {
    singletonContainerMountedAutomatically = true;
    (0, _env.mount)(SingletonContainer);
  }

  mountIntoContainer(hook);
};

exports.addHook = addHook;

var resetLocalStateForTests = function resetLocalStateForTests() {
  singletonContainerMounted = false;
  singletonContainerRendered = false;
  singletonContainerMountedAutomatically = false;
  mountQueue = [];
  mountIntoContainer = mountIntoContainerDefault;
};

exports.resetLocalStateForTests = resetLocalStateForTests;